% Active sonar.
% Compressive detection using known waveform basis.
% On detection, DOA estimates are obtained using the complex GMC method.
clear all;

speedOfSound = 1500;
samplingFrequency = 200000;

plotRows = 2;
plotColumns = 2;


%
% Add paths.
%
addpath('.\Solvers');
addpath('.\Bases');
addpath('.\Detectors');
addpath('.\Scenarios');
addpath('.\Sensors');
addpath('.\Waveforms');


%
% Define sensor system.
%
disp('Defining sensor system...');
noOfSensors = 8;
sensorSpacing = speedOfSound / (2*samplingFrequency);
sensorRadius = sensorSpacing / (2*sin(pi/noOfSensors));
sensorPositions = 8*CYL(noOfSensors, sensorRadius);
referenceSensorPositions = ULA(1, 0);


%
% Load scenario.
%
disp('Creating target solution...');
timeDuration = 8192 / samplingFrequency;
[targetData, targetDirections] = scenario09( samplingFrequency, timeDuration );
noOfTimeSamples = size(targetData, 1);


%
% Create sensor data.
%
disp('Creating sensor data...');
noiseLevel = -5;
sensorData = createSensorData( samplingFrequency, speedOfSound, sensorPositions, targetData, targetDirections, noiseLevel );
referenceSensorData = createSensorData( samplingFrequency, speedOfSound, referenceSensorPositions, targetData, targetDirections, noiseLevel );


%
% Create conventional beam data.
%
disp('Applying conventional beamforming...');
noOfBeams = 361;
beamDirections = linspace(-pi, pi, noOfBeams);
blockLength = 1024;
[beamData, broadbandData, duration] = createBeamData( sensorData, samplingFrequency, speedOfSound, sensorPositions, beamDirections, blockLength );

subplot(plotRows, plotColumns, 1);
imagesc(beamDirections*180/pi, (0:noOfTimeSamples-1), 10*log10(broadbandData));
title('Conventional beamformer');


%
% Create compressive beam data.
%
disp('Applying compressive beamforming...');
[cbeamData, cbroadbandData, cduration] = createCompressiveBeamData( sensorData, samplingFrequency, speedOfSound, sensorPositions, beamDirections, blockLength );

subplot(plotRows, plotColumns, 3);
imagesc(beamDirections*180/pi, (0:noOfTimeSamples-1), 10*log10(cbroadbandData));
title('Compressive beamformer');


%
% Create detection data.
%
disp('Applying compressive detector...');

alpha = 0.001;

blockLength = 256; %blockLength/2;
%noOfBlocks = ceil( (noOfTimeSamples-blockLength)*2 / blockLength + 1 );
noOfBlocks = floor( (noOfTimeSamples / blockLength) );

noOfDetectorMeasurements = blockLength/4;
detectorMeasurementBasis = orth(randn(blockLength, noOfDetectorMeasurements))';

startFrequency = 25000;
stopFrequency = 30000;
pulseDuration = timeDuration * 2/3;
k = ( 1 - startFrequency / stopFrequency ) / pulseDuration;

noOfSignatures = floor( k * pulseDuration * startFrequency * blockLength / samplingFrequency / (1 - k * pulseDuration) ) + 1;

SignatureBasis = zeros(noOfDetectorMeasurements, noOfSignatures);
SignatureDescriptor = zeros(2, noOfSignatures);
SignatureBasisSquares = zeros(2, noOfSignatures);

for signatureIndex = 1:noOfSignatures
    startTime = 1/k * 1 / (1 + startFrequency*blockLength/(samplingFrequency*(signatureIndex-1)));
    stopTime = startTime + blockLength / samplingFrequency;
    subStartFrequency = startFrequency / (1 - k*startTime);
    subStopFrequency = startFrequency / (1 - k*stopTime);
    
    SignatureBasis(:, signatureIndex) = detectorMeasurementBasis * HFM( subStartFrequency, subStopFrequency, samplingFrequency, blockLength, 0 ) / sqrt(blockLength);
    SignatureDescriptor(:, signatureIndex) = [subStartFrequency; (1-subStartFrequency/subStopFrequency)/(blockLength/samplingFrequency)];
end

SignatureBasisSquares(1, :) = sum(SignatureBasis.^2);
SignatureBasisSquares(2, :) = sum(abs(SignatureBasis).^2);

basisFrequencies = unique(SignatureDescriptor(1,:));
noOfBasisFrequencies = length(basisFrequencies);

detectionData = zeros(noOfBlocks, noOfBasisFrequencies);

noOfMeasurementsPerSensor = 10;
measurementBasis = orth(randn(blockLength, noOfMeasurementsPerSensor))';

beamDelays = sensorPositions' * [cos(beamDirections); sin(beamDirections)] / speedOfSound;

doaData = zeros(noOfBlocks, noOfBeams);

for blockIndex = 1:noOfBlocks
	% Obtain compressive reference data.
%    referenceSignal = referenceSensorData((blockIndex-1)*blockLength/2+1:(blockIndex-1)*blockLength/2+blockLength, 1);
    referenceSignal = referenceSensorData((blockIndex-1)*blockLength+1:blockIndex*blockLength, 1);
    ReferenceSignal = detectorMeasurementBasis * referenceSignal;

    % Perform detection.
    detections = getDetections( alpha, ReferenceSignal, SignatureBasis, SignatureBasisSquares, noOfDetectorMeasurements/blockLength );

    % Pursue all detections.
    for detectionIndex = 1:min(size(detections, 2), 5)
        basisIndex = detections(1,detectionIndex);

        pulse.type = 'HFM';
        pulse.amplitude = detections(3,detectionIndex);
        pulse.frequency = SignatureDescriptor(1, basisIndex);
        pulse.level = SignatureDescriptor(2, basisIndex);

        detectionData(blockIndex, basisIndex) = detections(2, detectionIndex);

        % Obtain DOA estimates.
%        blockData = sensorData((blockIndex-1)*blockLength/2+1:(blockIndex-1)*blockLength/2+blockLength, :);
        blockData = sensorData((blockIndex-1)*blockLength+1:blockIndex*blockLength, :);
        measurementData = measurementBasis * blockData;

        doaEstimate = getComplexDOAEstimate( measurementData, measurementBasis, beamDelays, samplingFrequency, 0.1, pulse );
        doaData(blockIndex, :) = doaData(blockIndex, :) + abs(doaEstimate');
    end
end

subplot(plotRows, plotColumns, 2);
imagesc(basisFrequencies, (0:noOfTimeSamples-1), abs(detectionData));
title('Compressive detector');

subplot(plotRows, plotColumns, 4);
imagesc(beamDirections*180/pi, (0:noOfTimeSamples-1), abs(doaData));
title('Direction-of-Arrival estimate');

% Remove paths.
addpath('.\Solvers');
rmpath('.\Bases');
rmpath('.\Detectors');
rmpath('.\Scenarios');
rmpath('.\Sensors');
rmpath('.\Waveforms');


return;



