function [ detections ] = getDetections( alpha, data, basis, basisSums, downsamplingFactor )
%GETDETECTIONS Computes (compressive) detections from complex basis.
% data                  - data vector of size Mx1 to search for detections.
% basis                 - complex basis of size MxL of possible signals.
% basisSums             - vector of size 2xL, first row is sum(basis.^2), second row is sum(abs(basis).^2).
% downsamplingFactor    - downsampling factor, equals M/N.
%
% detections            - vector of size 3xD: basis index, detection strength, amplitude
%

[M, L] = size(basis);

% Compute sums if needed.
if( nargin < 4 )
    basisSums = zeros(2, L);
    
    basisSums(1, :) = sum(basis.^2);
    basisSums(2, :) = sum(abs(basis).^2);
end

if ( nargin < 5 )
    downsamplingFactor = 1;
end

% Perform detection.
z = data' * basis;

amplitudes = 2 * z / downsamplingFactor;
signalNormsSquared = 0.5 * real( amplitudes.^2 .* basisSums(1,:) + abs(amplitudes).^2 .* basisSums(2,:) );
statistics = real(conj(amplitudes) .* z);
sigmas = real( sqrt( (1/M) * (norm(data)^2 + signalNormsSquared - 2*statistics) ) ) + eps;
gammas = sigmas .* sqrt(signalNormsSquared) * computeInverseQ(alpha);

detectionMask = statistics > gammas;
basisIndices = (1:L);
detectionIndices = basisIndices(detectionMask);
detectionStrengths = statistics(detectionMask);
detectionAmplitudes = amplitudes(detectionMask);

D = length(detectionStrengths);

[sortedDetectionStrengths, sortedDetectionIndices] = sort(detectionStrengths, 2, 'descend');
detections = zeros(3, D);

if ( D ~= 0 )
    detections(1, :) = detectionIndices(sortedDetectionIndices);
    detections(2, :) = sortedDetectionStrengths;
    detections(3, :) = detectionAmplitudes(sortedDetectionIndices);
end

end

