% Simulates passive sonar.
% Compressive detection using LFM basis.
% On detection, DOA estimates are obtained using the GMC method.
clear all;

speedOfSound = 1500;
samplingFrequency = 200000;

plotRows = 2;
plotColumns = 2;


%
% Add paths.
%
addpath('.\Solvers');
addpath('.\Bases');
addpath('.\Detectors');
addpath('.\Scenarios');
addpath('.\Sensors');
addpath('.\Waveforms');


%
% Define sensor system.
%
disp('Defining sensor system...');
noOfSensors = 8;
sensorSpacing = speedOfSound / (2*samplingFrequency);
sensorRadius = sensorSpacing / (2*sin(pi/noOfSensors));
sensorPositions = 8*CYL(noOfSensors, sensorRadius);
referenceSensorPositions = ULA(1, 0);


%
% Load scenario.
%
disp('Creating target solution...');
timeDuration = 8192 / samplingFrequency;
[targetData, targetDirections] = scenario01( samplingFrequency, timeDuration );
noOfTimeSamples = size(targetData, 1);


%
% Create sensor data.
%
disp('Creating sensor data...');
noiseLevel = 0;
sensorData = createSensorData( samplingFrequency, speedOfSound, sensorPositions, targetData, targetDirections, noiseLevel );
referenceSensorData = createSensorData( samplingFrequency, speedOfSound, referenceSensorPositions, targetData, targetDirections, noiseLevel );


%
% Create conventional beam data.
%
disp('Applying conventional beamforming...');
noOfBeams = 361;
beamDirections = linspace(-pi, pi, noOfBeams);
blockLength = 1024;
[beamData, broadbandData, duration] = createBeamData( sensorData, samplingFrequency, speedOfSound, sensorPositions, beamDirections, blockLength );

subplot(plotRows, plotColumns, 1);
imagesc(beamDirections*180/pi, (0:noOfTimeSamples-1), 10*log10(broadbandData));
title('Conventional beamformer');


%
% Create compressive beam data.
%
disp('Applying compressive beamforming...');
[cbeamData, cbroadbandData, cduration] = createCompressiveBeamData( sensorData, samplingFrequency, speedOfSound, sensorPositions, beamDirections, blockLength );

subplot(plotRows, plotColumns, 3);
imagesc(beamDirections*180/pi, (0:noOfTimeSamples-1), 10*log10(cbroadbandData));
title('Compressive beamformer');


%
% Create detection data.
%
disp('Applying compressive detector...');

alpha = 0.001;

blockLength = 256; %blockLength/2;
%noOfBlocks = ceil( (noOfTimeSamples-blockLength)*2 / blockLength + 1 );
noOfBlocks = floor( (noOfTimeSamples / blockLength) );

noOfDetectorMeasurements = blockLength/4;
detectorMeasurementBasis = orth(randn(blockLength, noOfDetectorMeasurements))';

[ SignatureBasis, SignatureDescriptor, SignatureBasisSquares ] = lfmBasis( samplingFrequency, blockLength, detectorMeasurementBasis );
noOfSignatures = size(SignatureBasis, 2);
basisFrequencies = unique(SignatureDescriptor(1,:));
noOfBasisFrequencies = length(basisFrequencies);

detectionData = zeros(noOfBlocks, noOfBasisFrequencies);

noOfMeasurementsPerSensor = 10;
measurementBasis = orth(randn(blockLength, noOfMeasurementsPerSensor))';

beamDelays = sensorPositions' * [cos(beamDirections); sin(beamDirections)] / speedOfSound;

doaData = zeros(noOfBlocks, noOfBeams);

for blockIndex = 1:noOfBlocks
	% Obtain compressive reference data.
%    referenceSignal = referenceSensorData((blockIndex-1)*blockLength/2+1:(blockIndex-1)*blockLength/2+blockLength, 1);
    referenceSignal = referenceSensorData((blockIndex-1)*blockLength+1:blockIndex*blockLength, 1);
    ReferenceSignal = detectorMeasurementBasis * referenceSignal;

    % Perform detection.
	detections = getDetections( alpha, ReferenceSignal, SignatureBasis, SignatureBasisSquares, noOfDetectorMeasurements/blockLength );
    if ( size(detections, 2) == 0 )
        continue;
    end

    % Pursue the largest detection.
    basisIndex = detections(1,1);

    pulse.type = 'LFM';
    pulse.amplitude = detections(3,1);
    pulse.frequency = SignatureDescriptor(1, basisIndex);
    pulse.level = SignatureDescriptor(2, basisIndex);

    centerFrequency = pulse.frequency * (1 + pulse.level * blockLength / samplingFrequency / 2);
    [r, detectionIndex] = min(abs(basisFrequencies - centerFrequency));
    detectionData(blockIndex, detectionIndex) = detections(2,1);

    % Obtain DOA estimates.
%    blockData = sensorData((blockIndex-1)*blockLength/2+1:(blockIndex-1)*blockLength/2+blockLength, :);
    blockData = sensorData((blockIndex-1)*blockLength+1:blockIndex*blockLength, :);
    measurementData = measurementBasis * blockData;

    doaEstimate = getDOAEstimate( measurementData, measurementBasis, beamDelays, samplingFrequency, 0.9, pulse );
    doaData(blockIndex, :) = abs(doaEstimate');
end

subplot(plotRows, plotColumns, 2);
imagesc(basisFrequencies, (0:noOfTimeSamples-1), log(abs(detectionData)));
title('Compressive detector');

subplot(plotRows, plotColumns, 4);
imagesc(beamDirections*180/pi, (0:noOfTimeSamples-1), log(doaData));
title('Direction-of-Arrival estimate');

% Remove paths.
rmpath('.\Solvers');
rmpath('.\Bases');
rmpath('.\Detectors');
rmpath('.\Scenarios');
rmpath('.\Sensors');
rmpath('.\Waveforms');




