function [ beamData, broadbandData, duration ] = createBeamData( sensorData, samplingFrequency, speedOfSound, sensorPositions, beamDirections, blockLength )
%CREATEBEAMDATA Creates beam data from sensor data.

noOfBeams = length(beamDirections);
[noOfTimeSamples, noOfSensors] = size(sensorData);

% Prepare for processing.
filterLength = 512;
totalLength = blockLength + filterLength;

noOfBlocks = ceil(noOfTimeSamples / blockLength);

frequencyIndices = [(0:totalLength/2)'; (1:totalLength/2-1)' - totalLength/2];
frequencies = frequencyIndices * samplingFrequency / totalLength;

beamDelays = sensorPositions' * [cos(beamDirections); sin(beamDirections)] / speedOfSound;

beamVectors = zeros(noOfSensors, noOfBeams, totalLength/2+1);

window = repmat([hann(filterLength); zeros(totalLength-filterLength, 1)], 1, noOfBeams);

for sensorIndex = 1:noOfSensors
    beamVector = exp(2*pi*1i*frequencies*(beamDelays(sensorIndex,:) - filterLength/(2*samplingFrequency))) / sqrt(noOfSensors);
    beamVector(totalLength/2+1,:) = 0;
    beamVector = fft(window .* ifft(beamVector));
    beamVectors(sensorIndex, :, :) = beamVector(1:totalLength/2+1,:).';
end


% Do conventional beamforming.
beamData = zeros(noOfTimeSamples, noOfBeams);

startIndex = 1;

tic;

for blockIndex = 1:noOfBlocks
    stopIndex = min(noOfTimeSamples, (blockIndex-1)*blockLength+totalLength);

    blockData = sensorData(startIndex:stopIndex, :);
    BlockData = fft(blockData, totalLength);

    BlockBeamData = zeros(totalLength, noOfBeams);

    for frequencyIndex = 2:totalLength/2
        BlockBeamData(frequencyIndex, :) = BlockData(frequencyIndex, :) * beamVectors(:, :, frequencyIndex);
        BlockBeamData(totalLength-frequencyIndex+2, :) = conj(BlockBeamData(frequencyIndex, :));
    end

    blockBeamData = real(ifft(BlockBeamData));

    stopIndex = min(noOfTimeSamples, blockIndex*blockLength);
    currentBlockLength = stopIndex - startIndex + 1;
    beamData(startIndex:stopIndex, :) = blockBeamData(filterLength:filterLength+currentBlockLength-1, :);

    startIndex = startIndex + blockLength;
end

duration = toc;

% Create conventional broadband plot.
noOfBlocks = ceil( (noOfTimeSamples-blockLength)*2 / blockLength + 1 );

broadbandData = zeros(noOfBlocks, noOfBeams);

for blockIndex = 1:noOfBlocks
    startIndex = (blockIndex-1)*blockLength/2+1;
    stopIndex = min(noOfTimeSamples, blockIndex*blockLength/2);

    broadbandData(blockIndex, :) = sum(abs(beamData(startIndex:stopIndex, :)).^2, 1);
end

end



