function [ beamData, broadbandData, duration ] = createCompressiveBeamData( sensorData, samplingFrequency, speedOfSound, sensorPositions, beamDirections, blockLength )
%CREATECOMPRESSIVEBEAMDATA Creates beamdata using compressive beam former.

noOfBeams = length(beamDirections);
[noOfTimeSamples, noOfSensors] = size(sensorData);

% Prepare for processing.
filterLength = 257;
totalLength = blockLength + filterLength - 1;

noOfBlocks = ceil(noOfTimeSamples / blockLength);

frequencyIndices = [(0:totalLength/2)'; (1:totalLength/2-1)' - totalLength/2];
frequencies = frequencyIndices * samplingFrequency / totalLength;

beamDelays = sensorPositions' * [cos(beamDirections); sin(beamDirections)] / speedOfSound;

svdBeamVectors = zeros(noOfSensors, noOfBeams, totalLength/2+1);
transformMatrix = zeros(noOfSensors, noOfSensors, totalLength/2+1);
ranks = zeros(totalLength/2+1, 1);

for frequencyIndex = 1:totalLength/2+1
    frequency = frequencies(frequencyIndex);
    beamVector = exp(2*pi*1i*frequency*beamDelays) / sqrt(noOfSensors);

    % Compute SVD variant.
    [U,S,V]  = svd(beamVector);

    rank = length( find(abs(diag(S)) > abs(S(1,1))*0.01) );

    U = U(:, 1:rank);
    S = S(1:rank, :);

    svdBeamVectors(1:rank, :, frequencyIndex) = U' * U * S * V';
    transformMatrix(1:rank, 1:noOfSensors, frequencyIndex) = U';
    ranks(frequencyIndex) = rank;    
end

% Do compressive beamforming
beamData = zeros(noOfTimeSamples, noOfBeams);

startIndex = 1;

tic;

for blockIndex = 1:noOfBlocks
    stopIndex = min(noOfTimeSamples, (blockIndex-1)*blockLength+totalLength);

    blockData = sensorData(startIndex:stopIndex, :);
    BlockData = fft(blockData, totalLength);

    BlockBeamData = zeros(totalLength, noOfBeams);

    for frequencyIndex = 2:totalLength/2
        rank = ranks(frequencyIndex);
        A = svdBeamVectors(1:rank, :, frequencyIndex);
        Y = transformMatrix(1:rank, 1:noOfSensors, frequencyIndex) * BlockData(frequencyIndex,:)';

        [solution, activeSet] = DCSSOMP(A, Y, 0.01);
        BlockBeamData(frequencyIndex, :) = solution';
        BlockBeamData(totalLength-frequencyIndex+2, :) = conj(BlockBeamData(frequencyIndex, :));
    end

    blockBeamData = real(ifft(BlockBeamData));

    stopIndex = min(noOfTimeSamples, blockIndex*blockLength);
    currentBlockLength = stopIndex - startIndex + 1;
    beamData(startIndex:stopIndex, :) = blockBeamData(filterLength:filterLength+currentBlockLength-1, :);

    startIndex = startIndex + blockLength;
end

duration = toc;

% Create broadband plot.
noOfBlocks = ceil( (noOfTimeSamples-blockLength)*2 / blockLength + 1 );

broadbandData = zeros(noOfBlocks, noOfBeams);

for blockIndex = 1:noOfBlocks
    startIndex = (blockIndex-1)*blockLength/2+1;
    stopIndex = min(noOfTimeSamples, blockIndex*blockLength/2);

    broadbandData(blockIndex, :) = sum(abs(beamData(startIndex:stopIndex, :)).^2, 1);
end

end

