function [ solution ] = getComplexDOAEstimate( measurementData, measurementBasis, beamDelays, samplingFrequency, alpha, pulse )
%GETDOAESTIMATE Computes DOA estimate based on compressive samples and reconstruction of waveform.
% measurementData       - Compressive measurements of sensor data (MxL).
% measurementBasis      - Measurement basis (MxN).
% beamDelays            - Delay of each sensor as a function of beam direction (LxB).
% alpha                 - Degree of reconstruction [0, 1].
% pulse                 - Structure with the following fields:
%   type
%   amplitude
%   frequency
%   level
%

[noOfMeasurementsPerSensor, blockLength] = size(measurementBasis);
[noOfSensors, noOfBeams] = size(beamDelays);

timeIndices = (0:blockLength-1)';
times = timeIndices / samplingFrequency;


% Construct system matrix by wavefrom reconstruction.
A = zeros(noOfMeasurementsPerSensor * noOfSensors, noOfBeams);

for sensorIndex = 1:noOfSensors
    sensorBeamDelays = beamDelays(sensorIndex, :);
    timeDelayMatrix = repmat(times, 1, noOfBeams) - repmat(sensorBeamDelays, blockLength, 1);

    switch pulse.type
        case 'LFM',
            sensorSignal = exp(2*pi*1i*pulse.frequency*(timeDelayMatrix + 0.5*pulse.level*timeDelayMatrix.^2)) / sqrt(blockLength);
        case 'HFM',
            if ( pulse.level == 0 )
                sensorSignal =  exp(2*pi*1i*pulse.frequency*timeDelayMatrix) / sqrt(blockLength);
            else
                sensorSignal = exp(-2*pi*1i*pulse.frequency/pulse.level * log(1 - pulse.level*timeDelayMatrix)) / sqrt(blockLength);
            end
    end

    A((sensorIndex-1)*noOfMeasurementsPerSensor+1:sensorIndex*noOfMeasurementsPerSensor, :) = measurementBasis * sensorSignal;
end


% Construct measurement vector.
Y = zeros(noOfMeasurementsPerSensor * noOfSensors, 1);
Y(1:end) = measurementData;


% Improve system matrix condition.
[U, S, V]  = svd(A);

rank = length( find(abs(diag(S)) > abs(S(1,1))*0.01) );
rank = max(rank, 2);

U = U(:, 1:rank);
S = S(1:rank, :);

A = U' * U * S * V';
Y = U' * Y;


% Obtain solution.
[solution, activeSet] = DCSSOMP(A, Y, alpha);
%solution(solution < 0) = 0;
%solution = abs(solution) / max(abs(solution));


end

